import re, subprocess
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway import sink_sms

command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')

def check(cmd, multiline):
    # print("Checking %s" % cmd)
    if cmd.lower() == 'sms' and multiline:
      return True
    else:
      return False


def run(lines):
    global IDENTIFIER, app_log

    IDENTIFIER = "SMS"
    app_log = setup_logging("sms-send")

    app_log.info("Forwarding SMS")
    toL = lines[1]
    m = re.match("To: ([0-9 +]+)", toL)
    if m:
      to = m.group(1).replace(' ', '')
      app_log.info("Matched To: %s" % to)
      msg = '\n'.join(lines[2:])
      app_log.debug("Sending SMS:\n%s" % msg)
      sink_sms.send_to(to, msg)
      ret = "Sent SMS to %s!" % to
    else:
      ret = "Couldn't match To: %s" % to
      app_log.error(ret)
    return ret

command_list.append({
    'name' : 'SMS-Forwarder',
    'check': check,
    'run': run
})



import os
from fbchat import Client
from fbchat.models import *
from smsgateway import sink_sms
from smsgateway.config import *
from smsgateway.sources.utils import *
from smsgateway.sources.facebook_utils import *

IDENTIFIER = "FB"

app_log = setup_logging("facebook")

# Subclass fbchat.Client and override required methods
class EchoBot(Client):
    def onMessage(self, mid, author_id, message, thread_id, thread_type, ts, metadata, msg, **kwargs):
        self.markAsDelivered(author_id, thread_id)
#        self.markAsRead(author_id)

        app_log.info("Message from {} in {} ({}): {}".format(author_id, thread_id, thread_type.name, message))
        source_name = author_id
        chat_info = {'ID' : mid}

        # If you're not the author, echo
        if author_id == self.uid:
            author_id = thread_id
            source_name = author_id
            isSent = True
        else:
            isSent = False

        for u in users:
            if str(u.uid) == str(author_id):
                source_name = u.name
                break
        if isSent:
            chat_info['To'] = source_name
        else:
            chat_info['From'] = source_name

        if thread_type == ThreadType.USER:
            chat_info['Type'] = 'User'
        elif thread_type == ThreadType.GROUP:
            chat_info['Type'] = 'Group'
        elif thread_type == ThreadType.ROOM:
            chat_info['Type'] = 'Channel'

        sink_sms.send_dict(IDENTIFIER, message, chat_info)
        app_log.info("Message sent by {name} to a {type} with ID {id}:\n{msg}".format(
            name='me' if isSent else source_name,
            type=chat_info['Type'],
            id=mid,
            msg=message
        ))

        # if isSent:
        #     print("To: %s\n\n%s\n" % (source_name, message))
        #     sink_sms.send_from_me(IDENTIFIER, message, source_name)
        # else:
        #     print("From: %s\n\n%s\n" % (source_name, message))
        #     sink_sms.send(IDENTIFIER, message, source_name)
#            self.sendMessage(message, thread_id=thread_id, thread_type=thread_type)

client = login(FB_CREDENTIALS, FB_COOKIE_PATH, EchoBot)

users = client.fetchAllUsers()

client.listen()



import asyncio, re, json
from async_generator import aclosing

from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway.sources.utils import *
from smsgateway import sink_sms

from telethon import TelegramClient, utils

from telethon.tl.types import Chat, User, Channel, \
  PeerUser, PeerChat, PeerChannel, \
  MessageMediaGeo, MessageMediaContact, MessageMediaPhoto, \
  MessageMediaDocument, MessageMediaWebPage, \
  Document, DocumentAttributeFilename, DocumentAttributeSticker


def check(cmd, multiline):
    global IDENTIFIER
    IDENTIFIER = "TG"
    # print("Checking %s" % cmd)
    if cmd.lower() == IDENTIFIER.lower() and multiline:
      return True
    else:
      return False

def get_display_name(entity):
    app_log.debug("Looking up entity " + entity.stringify())
    if isinstance(entity, User):
        return ' '.join([x for x in [entity.first_name, entity.last_name] if x])
    elif isinstance(entity, Chat) or isinstance(entity, Channel):
        return entity.title
    else:
        return None

async def send_message(message, to_matched):
    app_log.info("Starting client..")
    client = TelegramClient(session_path, api_id, api_hash)
    try:
        await client.start()
    except Exception as e:
        ret = "Could not connect! Run python3 -m smsgateway.sources.commands.send_telegram to authorize!\nError: %s" % e
        app_log.error(ret)
        return (False, ret)

    to = None
    async with aclosing(client.iter_dialogs()) as agen:
      async for x in agen:
          name = get_display_name(x.entity)
          if name and name == to_matched:
            to = x.entity.id
            app_log.info("Found it via display_name: %s" % x.entity.stringify())
            break
    if not to:
        app_log.warning(f"Couldn't find {to}! Trying directly..")
        to = name = to_matched

    app_log.info("Sending Telegram msg:\n%s" % message)

    try:
        import getpass
        app_log.info("I am: %s" % getpass.getuser())
    except:
        pass

    await client.send_message(to, message)
    await client.disconnect()
    msg = format_sms(IDENTIFIER, message, {
      'to': name,
      'status': 'Processed'
    })
    app_log.info(msg)
    # ret = '\n'.join([
    #   IDENTIFIER,
    #   f"To: {name}",
    #   "",
    #   message
    # ])
    return (True, msg)

def init():
    global app_log, IDENTIFIER, command_regex, api_id, api_hash, session_path
    app_log = setup_logging("telegram-send")
    IDENTIFIER = "TG"
    command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')

    api_id = 242101
    api_hash = "80cbc97ce425aae38c1e0291ef2ab2a4"

    session_path = os.path.join(CONFIG_DIR, 'telegram-send')

def run(lines):
    init()

    app_log.info("Forwarding Telegram Message")
    messageStarted = False
    to_matched = None
    message = ""

    for line in lines[1:]: # skip IDENTIFIER
        if messageStarted:
            message += f"\n{line}"
        elif not line.strip(): # empty line
            messageStarted = True
        else:
            mTo = re.match("^To: (.*)$", line)
            if mTo:
                to_matched = mTo.group(1).strip()
            else:
                app_log.warning(f"Unkown header: {line}!")

    if to_matched and message:
      loop = asyncio.get_event_loop()
      (success, ret) = loop.run_until_complete(send_message(message, to_matched))
      if success:
          ret = None
      loop.close()
    else:
      ret = f"Couldn't match To: {to_matched} or message {message}"
      app_log.error(ret)
    return ret


if __name__ == '__main__':
    init()
    client = TelegramClient(session_path, api_id, api_hash)
    if not client.start():
        app_log.error("Could not connect to Telegram!\nIf you haven't authorized this client, run python3 -m smsgateway.sources.commands.send_telegram!")
        sys.exit(1)

command_list.append({
    'name' : 'TG-Forwarder',
    'check': check,
    'run': run
})



import datetime, os
import smsgateway.config as conf
from smsgateway.sources.utils import *
from cryptography.fernet import Fernet

def send_dict(type, text, headers):
    send_to(conf.CONTROL_PHONES[0], format_sms(type, text, headers))
def send(type, text, _from, phone=None, group=None):
    lines = [
        type,
        f"From: {_from}"
    ]
    if group:
        lines += [f"Group: {group}"]
    if phone:
        lines += [f"Phone: {phone}"]
    if text:
        lines += [
            "",
            text
        ]
    send_to(conf.CONTROL_PHONES[0], '\n'.join(lines))

def send_from_me(type, text, to):
    lines = [
        type,
        f"To: {to}",
        "",
        text
    ]
    send_to(conf.CONTROL_PHONES[0], '\n'.join(lines))

def send_notif(text):
    send_to(conf.CONTROL_PHONES[0], text)

def send_to(to, text):
    print("Sending SMS:\n%s" % text)

    d = datetime.datetime.now().strftime("%y-%m-%d_%H-%M-%S.%f")[:-3]
    p = os.path.join(SMS_DIR, d)
    if conf.KEY:
        alphabet = "ISO"
        f = Fernet(conf.KEY)
        encrypted = f.encrypt(text.encode("utf-8"))
        msg = "%8%" + encrypted.decode("utf-8")
    else:
        alphabet = "UCS-2"
        msg = text
    header = '\n'.join([f"To: {to}", f"Alphabet: {alphabet}", "", ""])
    print("Writing sms to %s" % p)
    with open(p, 'wb') as f:
        f.write(header.encode("UTF-8"))
        f.write(msg.encode("UTF-8"))
        f.flush()



import os, sys
from appdirs import *
import importlib.util

KEY = None

CONFIG_DIR = user_config_dir(appname="smsgateway", appauthor="craeckie")
if not os.path.exists(CONFIG_DIR):
    os.mkdir(CONFIG_DIR)

_config_file = os.path.join(CONFIG_DIR, "config.py")
if not os.path.exists(_config_file):
    print("Error: no config file at %s!" % _config_file)
    sys.exit(1)

sys.path.append(CONFIG_DIR)
from config import *



from yowsup.layers.interface                           import YowInterfaceLayer, ProtocolEntityCallback
from yowsup.layers.protocol_messages.protocolentities  import TextMessageProtocolEntity
from yowsup.layers.protocol_receipts.protocolentities  import OutgoingReceiptProtocolEntity
from yowsup.layers.protocol_acks.protocolentities      import OutgoingAckProtocolEntity
from yowsup.layers.protocol_groups.protocolentities    import AddParticipantsIqProtocolEntity
from smsgateway import sink_sms
from smsgateway.config import *

IDENTIFIER = "WA"

class EchoLayer(YowInterfaceLayer):
    def __init__(self):
        super(EchoLayer, self).__init__()
        self.jidAliases = WA_USER_ALIASES
        self.gidAliases = WA_GROUP_ALIASES

    def jidToAlias(self, jid):
        for ajid, alias in self.jidAliases.items():
            if ajid == jid:
                return alias
        return jid
    def gidToAlias(self, jid):
        for ajid, alias in self.gidAliases.items():
            if ajid == jid:
                return alias
        return jid

    @ProtocolEntityCallback("message")
    def onMessage(self, messageProtocolEntity):
        #send receipt otherwise we keep receiving the same message over and over

        if True:
            print("Sending receipt")
            receipt = OutgoingReceiptProtocolEntity(messageProtocolEntity.getId(), messageProtocolEntity.getFrom(), 'read', messageProtocolEntity.getParticipant())

            #outgoingMessageProtocolEntity = TextMessageProtocolEntity(
            #    messageProtocolEntity.getBody(),
            #    to = messageProtocolEntity.getFrom())

            #self.toLower(receipt)
            #self.toLower(outgoingMessageProtocolEntity)
            t = messageProtocolEntity.getBody()
            f = messageProtocolEntity.getFrom(False)
            print("From: %s" % f)
            if f == WA_NUMBER:
               print("Message from myself!")
            s = f.split('-')[0]
            print("Split: %s" % s)
            alias = self.jidToAlias(s)
            print("Alias: %s" % alias)
            if messageProtocolEntity.isGroupMessage():
                print("Is group message")
                g = f.split('-')[1]
                g_alias = self.gidToAlias(g)
                alias += '@' + g_alias
            print("From: %s\nMessage: %s" % (alias, t))
            sink_sms.send(IDENTIFIER, t, f)

    @ProtocolEntityCallback("receipt")
    def onReceipt(self, entity):
        ack = OutgoingAckProtocolEntity(entity.getId(), "receipt", entity.getType(), entity.getFrom())
        print("Receipt: %s" % ack)
        self.toLower(ack)



import time, asyncio
from smsgateway import sink_sms
from smsgateway.config import *
from smsgateway.sources.utils import *

from slackclient import SlackClient

IDENTIFIER = "SL"
app_log = setup_logging("slack")

sc = SlackClient(SL_TOKEN)


def main():
    data = sc.api_call("users.list")
    users = {}
    app_log.info("Getting users..")
    if not data['ok']:
        app_log.error("Could not get user information!")
        app_log.error(f"Token: {SL_TOKEN}")
        app_log.error(data)
        return
    for u in data['members']:
        uID = u['id']
        uName = u['real_name']
        app_log.debug(f"{uID}: {uName}")
        users[uID] = uName
    app_log.info(f"Got {len(users)} users")

    app_log.info("Getting channels..")
    data = sc.api_call("channels.list")
    channels = {}
    if not data['ok']:
        app_log.error("Could not get channel information!")
        app_log.error(f"Token: {SL_TOKEN}")
        app_log.error(data)
        return
    for u in data['channels']:
        uID = u['id']
        uName = u['name']
        app_log.debug(f"{uID}: {uName}")
        channels[uID] = uName
    app_log.info(f"Got {len(channels)} channels")


    if not sc.rtm_connect(auto_reconnect=True):
        app_log.error("Could not connect to Slack!")
        return False

    while True:
        data = sc.rtm_read()
        if data:
            for entry in data:
                app_log.debug(entry)
                type = entry['type']
                if type == 'message':
                    text = None
                    if 'message' in entry:
                        msg = entry['message']
                        isInChannel = True
                        app_log.info("Got a message in a channel!")
                    elif 'text' in entry:
                        msg = entry
                        isInChannel = False
                        app_log.info("Got a message from a user/bot!")
                    else:
                        app_log.warning("Could not parse this:\n" + entry)
                        continue
                    if not msg['text']:
                        print("Has no message!")
                        continue
                    else:
                        text = msg['text']
                        user = users[msg['user']]
                        chat_info = {
                            'date': str(int(float(msg['ts']))),
                            'from': user
                        }
                        if 'client_msg_id' in msg:
                            chat_info['ID'] = msg['client_msg_id']
                        if isInChannel:
                            channel = channels[entry['channel']]
                            chat_info.update({
                              'type': 'Group',
                              'to': channel,
                            })
                        sink_sms.send_dict(IDENTIFIER, text, chat_info)

        time.sleep(1)



if __name__ == '__main__':
    main()
    # loop = asyncio.get_event_loop()
    # loop.run_until_complete(main())



import email, time, argparse
from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

from imapclient import IMAPClient
from email.header import decode_header, make_header
from email.utils import parseaddr
from bs4 import BeautifulSoup

IDENTIFIER = "EM"

app_log = None

def parse_part(part):
    payload = None
    if part.get_content_maintype() == 'text':
        type = part.get_content_subtype()
        charset = 'UTF-8'
        charsets = part.get_charsets()
        if len(charsets) > 0:
            charset = charsets[0]
        app_log.debug(f"Decoding payload with charset {charset}")
        payload = part.get_payload(decode=True).decode(charset)
        if type == 'html':
            b = BeautifulSoup(payload, 'html.parser')
            for s in b(["script", "style"]):
                s.decompose()
            text = b.get_text()
            lines = (line.strip() for line in text.splitlines())
            chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
            text = '\n'.join(chunk for chunk in chunks if chunk)
            payload = text
    return payload

def parse_fetch(fetch):
    msg = email.message_from_bytes(fetch)
    body = None
    attachment = None
    if msg.is_multipart():
        for part in msg.get_payload():
            payload = parse_part(part)
            if payload:
              if part.get_content_disposition() == 'attachment':
                  name = part.get_filename()
                  attachment = "Attachment: %s\n\n%s" % (name, payload)
              else:
                  body = payload
    else:
        body = parse_part(msg)
    if body and attachment:
        body += "\n\n----\n\n%s" % attachment
    elif not body:
        body = "No body!"
    items = {'Body': body}
    for name in ['Subject', 'From']:
        items[name] = make_header(decode_header(msg[name]))
    to_addr = parseaddr(msg['To'])
    to = to_addr[len(to_addr) - 1]
    if not to:
        to = make_header(decode_header(msg['To']))
    items['To'] = to

    return items
def fetch_messages(server, num):
    app_log.info("Fetching %s messages" % num)
    msg_list = server.sort(sort_criteria='REVERSE DATE', charset='UTF-8')
    app_log.debug("Found %s messages" % len(msg_list))
    last_messages = msg_list[0:num]
    msg_fetches = server.fetch(last_messages, ['RFC822'])
    app_log.info("Fetched %s messages" % len(msg_fetches))
    messages = []
    for msg_id in last_messages:
        app_log.debug(msg_id)
        fetch = msg_fetches[msg_id]
        if b'RFC822' in fetch:
            msg = parse_fetch(fetch[b'RFC822'])
            messages.append(msg)
            for k, v in msg.items():
                app_log.debug("%s: %s" % (k, v))
        else:
            app_log.warning("No RFC822 in this fetch: %s" % fetch)
    return messages

def forward_messages(messages):
    for msg in messages:
        body = msg['Body']
        del msg['Body']
        app_log.debug("Forwarding message: %s" % str(msg))
        sink_sms.send_dict(IDENTIFIER, body, msg)

def login(name):
    account = EMAIL_ACCOUNTS[name]
    server = IMAPClient(account['Host'])
    server.login(account['User'], account['Password'])
    inbox = server.select_folder('INBOX', readonly=True)
    app_log.debug("Inbox: %s" % inbox)
    return server

def wait_idle(server, account):
    last_exists = 0
    unseen = server.folder_status('INBOX', what=b'UNSEEN')[b'UNSEEN']
    app_log.info("Waiting for IDLE response..")
    server.idle()
    loop_start = time.time()
    while True:
        # noop = server.noop()
        # app_log.debug("Server returned %s" % noop)
        app_log.debug("Checking IDLE")
        start = time.time()
        responses = server.idle_check(timeout=10)
        diff = time.time() - start
        app_log.debug("Server sent: %s" % responses if responses else "nothing")
        if responses:
            num = 0
            for resp in responses:
              if len(resp) >= 2:
                n = resp[0]
                type = resp[1]
                if type == b'EXISTS':
                  if n > last_exists:
                    num += 1
                  last_exists = n
            if num > 0:
              try:
                  server.idle_done()
              except:
                  pass
              messages = fetch_messages(server, num)
              forward_messages(messages)
              server.idle()
        elif diff < 5:
            app_log.warning(f"IDLE check took only {diff} seconds!\nNeed to reconnect..")
            server = login(account)
        elif time.time() - loop_start >= 60:
            # Still connected but IDLE empty -> check if really no change
            app_log.debug("Checking if unseen messages increased..")
            try:
                server.idle_done()
            except:
                pass
            new_unseen = server.folder_status('INBOX', what=b'UNSEEN')[b'UNSEEN']
            diff = new_unseen - unseen
            if diff > 0:
              app_log.info(f"Found {diff} unseen messages!")
              messages = fetch_messages(server, diff)
              forward_messages(messages)
              unseen = new_unseen
            else:
              app_log.debug(f"No new unseen messages: {new_unseen} = {unseen}")
            server.idle()

        if time.time() - loop_start > 60*15:
            app_log.info("IDLE timeout elapsed, re-establishing connection..")
            try:
                server.idle_done()
            except:
                pass
            try:
                server.logout()
            except:
                pass
            server = login(account)
            loop_start = time.time()

    server.idle_done()

def main():
    global app_log
    parser = argparse.ArgumentParser()
    parser.add_argument("account", help="The account name from config.py")
    args = parser.parse_args()

    account = args.account

    app_log = setup_logging("email-%s" % account)


    app_log.info("Logging in..")
    server = login(account)

    wait_idle(server, account)

    server.logout()

if __name__ == '__main__':
    main()
